<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Searcher for Elementor
 * Customizable search bar for Elementor editor
 * Exclusively on https://1.envato.market/searcher-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.2
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\SearcherElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\SearcherElementor\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Searcher - Custom Elementor Widget.
 **/
class searcher_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
     'mdp-searcher-elementor-admin',
        UnityPlugin::get_url() . 'src/Merkulove/Unity/assets/css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version()
        );
        wp_register_style(
            'mdp-searcher-elementor',
                UnityPlugin::get_url() . 'css/searcher-elementor' . UnityPlugin::get_suffix() . '.css',
                    [], UnityPlugin::get_version()
        );
	    wp_register_script(
            'mdp-searcher-elementor',
               UnityPlugin::get_url() . 'js/searcher-elementor' . UnityPlugin::get_suffix() . '.js',
                   [ 'jquery' ],
                   UnityPlugin::get_version(), true
        );

        /** Pass variables to script */
        wp_localize_script( 'mdp-searcher-elementor', 'mdpSearcherObjectElementorWP', [
            'endpoint' => admin_url( 'admin-ajax.php' ),
            'nonce' => wp_create_nonce( 'mdp-searcher-elementor-nonce' )
        ] );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-searcher-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Searcher', 'searcher-elementor' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-searcher-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'general' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Searcher' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-searcher-elementor', 'mdp-searcher-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-searcher-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> General Content Section. */
        $this->section_content_general();

        /** Content -> Button Content Section. */
        $this->section_search_button();

        /** Content -> Search Bar Label Content Section. */
        $this->section_search_bar_label();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Search Form. */
        $this->section_style_search_form();

        /** Style -> Section Style Search Button. */
        $this->section_style_search_button();

        /** Style -> Section Style Search Field. */
        $this->section_style_search_field();

        /** Style -> Section Style Label. */
        $this->section_style_label();

        /** Style -> Section Style Result Box. */
        $this->section_style_result_box();

        /** Style -> Section Style Result Item. */
        $this->section_style_result_item();

        /** Style -> Section Style Result Item Post Thumbnail. */
        $this->section_style_result_item_thumbnail();

        /** Style -> Section Style Result Item Post Title. */
        $this->section_style_result_post_title();

        /** Style -> Section Style Result Item Post Excerpt. */
        $this->section_style_result_post_excerpt();

    }

    /**
     * Get post types for select options.
     *
     * @return array
     * @since 1.0.0
     * @access private
     *
     */
    private function get_post_types_options() {
        $types = wp_list_pluck( get_post_types( [ 'public' => true, 'show_in_nav_menus' => true ], 'objects' ), 'label', 'name' );
        return array_diff_key( $types, ['elementor_library', 'attachment'] );
    }


    /**
     * Add widget controls: Content -> General Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_general() {

        $this->start_controls_section( 'section_content_general', [
            'label' => esc_html__( 'General', 'searcher-elementor' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'search_form_alignment',
            [
                'label' => esc_html__( 'Alignment', 'searcher-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'searcher-elementor' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'searcher-elementor' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'searcher-elementor' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-searcher-elementor-box' => 'justify-content: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );


        $this->add_control(
            'search_bar_placeholder',
            [
                'label' => esc_html__( 'Placeholder', 'searcher-elementor' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type your placeholder here', 'searcher-elementor' ),
            ]
        );

        $this->add_control(
            'request_type',
            [
                'label'     => esc_html__( 'Request type', 'searcher-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'async'    => esc_html__( 'Asynchronous', 'searcher-elementor' ),
                    'regular'    => esc_html__( 'Regular', 'searcher-elementor' ),
                ],
                'default'   => 'async',
                'separator' => 'none',
            ]
        );


	    $this->add_control(
		    'show_search_button',
		    [
			    'label' => esc_html__( 'Show search button', 'searcher-elementor' ),
			    'type' => Controls_Manager::SWITCHER,
			    'return_value' => 'yes',
			    'default' => 'yes',
		    ]
	    );

        $this->add_control(
            'search_post_types',
            [
                'label' => esc_html__( 'Search post types', 'searcher-elementor' ),
                'type' => Controls_Manager::SELECT2,
                'multiple' => true,
                'options' => $this->get_post_types_options(),
                'default' => ['post'],
            ]
        );

        $this->add_control(
            'show_results_on_typing',
            [
                'label' => esc_html__( 'Show results on typing', 'searcher-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 10,
                        'step' => 1,
                    ],
                ],
                'condition' => [
                    'request_type' => 'async'
                ]
            ]
        );

        $this->add_control(
            'request_delay',
            [
                'label' => esc_html__( 'Request delay', 'searcher-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 100,
                        'max' => 900,
                        'step' => 50,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 100
                ],
                'condition' => [
                    'request_type' => 'async'
                ]
            ]
        );

        $this->add_control(
            'empty_result_text',
            [
                'label' => esc_html__( 'Empty result text', 'searcher-elementor' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'placeholder' => esc_html__( 'Type your empty result text here', 'searcher-elementor' ),
                'condition' => [
                    'request_type' => 'async'
                ]
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Search Button Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_search_button() {

        $this->start_controls_section( 'section_search_button', [
            'label' => esc_html__( 'Search button', 'searcher-elementor' ),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'show_search_button' => 'yes'
            ]
        ] );

        $this->add_control(
            'search_button_content',
            [
                'label' => esc_html__( 'Search button content', 'searcher-elementor' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'default' => 'Search',
                'placeholder' => esc_html__( 'Type your search button content here', 'searcher-elementor' ),
            ]
        );

        $this->add_control(
            'search_button_position',
            [
                'label'     => esc_html__( 'Search button position', 'searcher-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'left'    => esc_html__( 'Left', 'searcher-elementor' ),
                    'right'    => esc_html__( 'Right', 'searcher-elementor' ),
                    'above-left'    => esc_html__( 'Above left', 'searcher-elementor' ),
                    'above-center'    => esc_html__( 'Above center', 'searcher-elementor' ),
                    'above-right'    => esc_html__( 'Above right', 'searcher-elementor' ),
                    'under-left'    => esc_html__( 'Under left', 'searcher-elementor' ),
                    'under-center'    => esc_html__( 'Under center', 'searcher-elementor' ),
                    'under-right'    => esc_html__( 'Under right', 'searcher-elementor' ),
                    'inside'    => esc_html__( 'Inside', 'searcher-elementor' )
                ],
                'default'   => 'right',
                'separator' => 'none',
            ]
        );

        $this->add_control(
            'show_search_button_icon',
            [
                'label' => esc_html__( 'Show search button icon', 'searcher-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'search_button_icon',
            [
                'label' => esc_html__( 'Search button icon', 'searcher-elementor' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-search',
                    'library' => 'solid',
                ],
                'condition' => [
                    'show_search_button_icon' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'search_button_icon_position',
            [
                'label' => esc_html__( 'Search button icon position', 'searcher-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'before',
                'options' => [
                    'before'  => esc_html__( 'Left', 'searcher-elementor' ),
                    'after' => esc_html__( 'Right', 'searcher-elementor' ),
                    'above-left' => esc_html__( 'Top left', 'searcher-elementor' ),
                    'above-center' => esc_html__( 'Top center', 'searcher-elementor' ),
                    'above-right' => esc_html__( 'Top right', 'searcher-elementor' ),
                    'under-left' => esc_html__( 'Bottom left', 'searcher-elementor' ),
                    'under-center' => esc_html__( 'Bottom center', 'searcher-elementor' ),
                    'under-right' => esc_html__( 'Bottom right', 'searcher-elementor' )
                ],
                'condition' => [
                    'show_search_button_icon' => 'yes'
                ]
            ]
        );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Content -> Search Bar Label Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_search_bar_label() {

        $this->start_controls_section( 'section_search_bar_label', [
            'label' => esc_html__('Search bar label', 'searcher-elementor'),
            'tab' => Controls_Manager::TAB_CONTENT,
        ] );

        $this->add_control(
            'search_bar_label',
            [
                'label' => esc_html__( 'Search bar label', 'searcher-elementor' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'placeholder' => esc_html__( 'Type your search bar label here', 'searcher-elementor' ),
            ]
        );

        $this->add_control(
            'search_bar_label_position',
            [
                'label'     => esc_html__( 'Search bar label position', 'searcher-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'left'    => esc_html__( 'Left', 'searcher-elementor' ),
                    'right'    => esc_html__( 'Right', 'searcher-elementor' ),
                    'above-left'    => esc_html__( 'Above left', 'searcher-elementor' ),
                    'above-center'    => esc_html__( 'Above center', 'searcher-elementor' ),
                    'above-right'    => esc_html__( 'Above right', 'searcher-elementor' ),
                    'below-left'    => esc_html__( 'Below left', 'searcher-elementor' ),
                    'below-center'    => esc_html__( 'Below center', 'searcher-elementor' ),
                    'below-right'    => esc_html__( 'Below right', 'searcher-elementor' ),
                ],
                'default'   => 'left',
                'separator' => 'none',
            ]
        );

        $this->add_control(
            'hide_search_bar_label',
            [
                'label' => esc_html__( 'Hide label', 'searcher-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @param array $default_padding
     * @param array $default_margin
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_margin_padding_controls( $section_id, $html_class, $default_padding = [], $default_margin = [] ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'searcher-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_margin,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'searcher-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'default' => $default_padding,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ?
                $opts['html_class'] : '',
            'active_class' => array_key_exists( 'active_class', $opts ) ?
                $opts['active_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ?
                $opts['include_color'] : true,
            'include_border' => array_key_exists( 'include_border', $opts ) ?
                $opts['include_border'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ?
                $opts['include_color'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ?
                $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ?
                $opts['include_transition'] : true,
            'additional_color' => array_key_exists( 'additional_color', $opts ) ?
                $opts['additional_color'] : false,
            'additional_background' => array_key_exists( 'additional_background', $opts ) ?
                $opts['additional_background'] : false,
            'include_outline' => array_key_exists( 'include_outline', $opts ) ?
                $opts['include_outline'] : false,
            'typography_name' => array_key_exists( 'typography_name', $opts ) ?
                $opts['typography_name'] : 'Typography',
            'additional_background_name' => array_key_exists( 'additional_background_name', $opts ) ?
                $opts['additional_background_name'] : '',
            'additional_background_class' => array_key_exists( 'additional_background_class', $opts ) ?
                $opts['additional_background_class'] : '',
            'additional_border_radius_class' => array_key_exists( 'additional_border_radius_class', $opts ) ?
                $opts['additional_border_radius_class'] : '',
            'additional_border_radius_hover_class' => array_key_exists( 'additional_border_radius_hover_class', $opts ) ?
                $opts['additional_border_radius_hover_class'] : '',
            'include_active_tab' => array_key_exists( 'include_active_tab', $opts ) ?
                $opts['include_active_tab'] : false,
            'active_tab_name' => array_key_exists( 'active_tab_name', $opts ) ?
                $opts['active_tab_name'] : 'FOCUS',
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ?
                $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ?
                $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ?
                $opts['color_hover_class'] : '',
            'color_active_class' => array_key_exists( 'color_active_class', $opts ) ?
                $opts['color_active_class'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ?
                $opts['color_hover_selector'] : '',
            'additional_color_name' => array_key_exists( 'additional_color_name', $opts ) ?
                $opts['additional_color_name'] : '',
            'additional_color_class' => array_key_exists( 'additional_color_class', $opts ) ?
                $opts['additional_color_class'] : '',
            'additional_color_hover_class' => array_key_exists( 'additional_color_hover_class', $opts ) ?
                $opts['additional_color_hover_class'] : '',
            'additional_color_active_class' => array_key_exists( 'additional_color_active_class', $opts ) ?
                $opts['additional_color_active_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ?
                $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ?
                $opts['typography_class'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ?
                $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ?
                $opts['additional_color_scheme_default'] : Color::COLOR_3
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__( $style_opts['typography_name'], 'searcher-elementor' ),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab(
            $section_id.'_normal_style_tab',
            ['label' => esc_html__( 'NORMAL', 'searcher-elementor' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'searcher-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'searcher-elementor' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_class'] => 'color: {{VALUE}} !important;',

                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'searcher-elementor'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
                ]
            );

        }

        if ( $style_opts['additional_background'] ) {

            $this->add_control(
                $section_id . '_' . $style_opts['additional_background_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_background_name'], 'searcher-elementor' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_background_class'] => 'background: {{VALUE}} !important;',

                    ],
                ]
            );

        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        if ( $style_opts['include_border'] ) {

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_normal',
                    'label' => esc_html__('Border Type', 'searcher-elementor'),
                    'selector' => "{{WRAPPER}} ." . $style_opts['html_class'],
                ]
            );

        }

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'searcher-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}; -webkit-border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'searcher-elementor' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab(
            $section_id.'_hover_style_tab',
            ['label' => esc_html__( 'HOVER', 'searcher-elementor' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'searcher-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'searcher-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'searcher-elementor'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
                ]
            );
        }

        if ( $style_opts['additional_background'] ) {

            $this->add_control(
                $section_id . '_' . $style_opts['additional_background_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_background_name'], 'searcher-elementor' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_background_class'].':hover' => 'background: {{VALUE}} !important;',

                    ],
                ]
            );

        }


        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'searcher-elementor' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        if ( $style_opts['include_border'] ) {

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_hover',
                    'label' => esc_html__('Border Type', 'searcher-elementor'),
                    'selector' => "{{WRAPPER}} ." . $style_opts['html_class'] . ":hover",
                ]
            );

        }

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'searcher-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_hover_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'searcher-elementor' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->end_controls_tab();

        if ( $style_opts['include_active_tab'] ) {

            $this->start_controls_tab(
                $section_id . '_focus_style_tab',
                ['label' => esc_html__( $style_opts['active_tab_name'], 'searcher-elementor' )]
            );

            if ( $style_opts['include_color'] ) {
                $this->add_control(
                    $section_id . '_active_color',
                    [
                        'label' => esc_html__( $style_opts['color_prefix'] . 'Color', 'searcher-elementor '),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['additional_color'] ) {
                $this->add_control(
                    $section_id . '_' . $style_opts['additional_color_name'] . '_active_text_color',
                    [
                        'label' => esc_html__( $style_opts['additional_color_name'], 'searcher-elementor' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['additional_color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['additional_color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['include_bg'] ) {
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name' => $section_id . '_background_active',
                        'label' => esc_html__( 'Background type', 'searcher-elementor' ),
                        'types' => ['classic', 'gradient', 'video'],
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );
            }

            if ( $style_opts['include_outline'] ) {
                $this->add_control(
                    $section_id . '_outline_style',
                    [
                        'label' => esc_html__( 'Outline', 'searcher-elementor' ),
                        'type' => Controls_Manager::SELECT,
                        'default' => '',
                        'options' => [
                            'none' => esc_html__( 'None', 'searcher-elementor' ),
                            '' => esc_html__( 'Default', 'searcher-elementor' ),
                            'auto' => esc_html__( 'Auto', 'searcher-elementor' ),
                            'solid'  => esc_html__( 'Solid', 'searcher-elementor' ),
                            'dotted' => esc_html__( 'Dotted', 'searcher-elementor' ),
                            'double' => esc_html__( 'Double', 'searcher-elementor' ),
                            'dashed' => esc_html__( 'Dashed', 'searcher-elementor' ),
                            'groove' => esc_html__( 'Groove', 'searcher-elementor' ),
                            'ridge' => esc_html__( 'Ridge', 'searcher-elementor' ),
                            'inset' => esc_html__( 'Inset', 'searcher-elementor' ),
                            'outset' => esc_html__( 'Outset', 'searcher-elementor' ),
                        ],
                        'selectors' => [
                            '{{WRAPPER}} .' . $style_opts['html_class'] . ':focus' => 'outline-style: {{VALUE}};',
                        ],
                    ]
                );

                $this->add_control(
                    $section_id . '_outline_color',
                    [
                        'label' => esc_html__( 'Outline color', 'searcher-elementor' ),
                        'type' => Controls_Manager::COLOR,
                        'condition' => [
                            $section_id . '_outline_style!' => 'none'
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['html_class'] . ':focus' => 'outline-color: {{VALUE}} !important;',
                        ],
                    ]
                );

                $this->add_control(
                     $section_id . '_custom_outline_width',
                    [
                        'label' => esc_html__( 'Custom outline width', 'searcher-elementor' ),
                        'type' => Controls_Manager::SWITCHER,
                        'return_value' => 'yes',
                        'default' => 'no',
                        'condition' => [
                            $section_id . '_outline_style!' => 'none'
                        ],
                    ]
                );

                $this->add_control(
                    $section_id . '_outline_width',
                    [
                        'label' => esc_html__( 'Outline width', 'searcher-elementor' ),
                        'type' => Controls_Manager::SELECT,
                        'default' => '',
                        'options' => [
                            '' => esc_html__( 'Default', 'searcher-elementor' ),
                            'thin' => esc_html__( 'Thin', 'searcher-elementor' ),
                            'medium'  => esc_html__( 'Medium', 'searcher-elementor' ),
                            'thick' => esc_html__( 'Thick', 'searcher-elementor' ),
                        ],
                        'condition' => [
                            $section_id . '_custom_outline_width!' => 'yes',
                            $section_id . '_outline_style!' => 'none'
                        ],
                        'selectors' => [
                            '{{WRAPPER}} .' . $style_opts['html_class'] . ':focus' => 'outline-width: {{VALUE}};',
                        ],
                    ]
                );

                $this->add_responsive_control(
                    $section_id . '_outline_custom_width',
                    [
                        'label' => esc_html__( 'Outline width', 'searcher-elementor' ),
                        'type' => Controls_Manager::SLIDER,
                        'size_units' => [ 'px', 'em' ],
                        'range' => [
                            'px' => [
                                'min' => 0,
                                'max' => 500,
                                'step' => 1,
                            ],
                            'em' => [
                                'min' => 0.1,
                                'max' => 100,
                                'step' => 0.1
                            ]
                        ],
                        'condition' => [
                            $section_id . '_custom_outline_width' => 'yes',
                            $section_id . '_outline_style!' => 'none'
                        ],
                        'selectors' => [
                            '{{WRAPPER}} .' . $style_opts['html_class'] . ':focus' => 'outline-width: {{SIZE}}{{UNIT}};',
                        ],
                    ]
                );


            }

            $this->add_control(
                $section_id . '_separate_active',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );

            if ( $style_opts['include_border'] ) {

                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => $section_id . '_border_active',
                        'label' => esc_html__('Border Type', 'searcher-elementor'),
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );

            }

            $this->add_responsive_control(
                $section_id . '_border_radius_active',
                [
                    'label' => esc_html__( 'Border radius', 'searcher-elementor' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => ['px', '%', 'em'],
                    'selectors' => [
                        "{{WRAPPER}} ." . $style_opts['active_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_active',
                    'label' => esc_html__( 'Box Shadow', 'searcher-elementor' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->end_controls_tab();

        }


        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Search Form.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_search_form() {

        $this->start_controls_section( 'section_style_search_form', [
            'label' => esc_html__( 'Search form', 'searcher-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
        ] );

        $this->generate_margin_padding_controls(
            'section_style_search_form',
            'mdp-searcher-elementor-search-form'
        );

        $this->add_responsive_control(
            'search_form_width',
            [
                'label' => esc_html__( 'Width', 'searcher-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-searcher-elementor-search-form' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_search_form', [
            'html_class' => 'mdp-searcher-elementor-search-form',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Search Button.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_search_button() {

        $this->start_controls_section( 'section_style_search_button', [
            'label' => esc_html__( 'Search button', 'searcher-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'show_search_button' => 'yes'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_search_button',
            'mdp-searcher-elementor-search-button'
        );

        $this->add_control(
            'search_button_icon_size_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_responsive_control(
            'search_button_icon_spacing',
            [
                'label' => esc_html__( 'Icon spacing', 'searcher-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'condition' => [
                    'show_search_button_icon' => 'yes'
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-searcher-elementor-button-icon" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'search_button_icon_size',
            [
                'label' => esc_html__( 'Icon size', 'searcher-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'condition' => [
                    'show_search_button_icon' => 'yes'
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-searcher-elementor-button-icon' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-searcher-elementor-button-icon i' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-searcher-elementor-button-icon svg' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};'
                ],
            ]
        );

        $this->add_responsive_control(
            'search_button_width',
            [
                'label' => esc_html__( 'Width', 'searcher-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-searcher-elementor-search-button' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'search_button_height',
            [
                'label' => esc_html__( 'Height', 'searcher-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-searcher-elementor-search-button' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_search_button', [
            'html_class' => 'mdp-searcher-elementor-search-button',
            'additional_color' => true,
            'include_active_tab' => true,
            'additional_color_name' => 'Icon color',
            'active_class' => 'mdp-searcher-elementor-search-button:focus',
            'typography_class' => 'mdp-searcher-elementor-search-button',
            'color_class' => 'mdp-searcher-elementor-search-button',
            'color_active_class' => 'mdp-searcher-elementor-search-button:focus',
            'additional_color_class' => 'mdp-searcher-elementor-search-button .mdp-searcher-elementor-button-icon',
            'additional_color_hover_class' => 'mdp-searcher-elementor-search-button:hover .mdp-searcher-elementor-button-icon',
            'color_hover_class' => 'mdp-searcher-elementor-search-button:hover',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-searcher-elementor-button-icon'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Search Button.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_search_field() {
        $this->start_controls_section( 'section_style_search_field', [
            'label' => esc_html__( 'Search field', 'searcher-elementor' ),
            'tab' => Controls_Manager::TAB_STYLE,
        ] );

        $this->generate_margin_padding_controls(
            'section_style_search_field',
            'mdp-searcher-elementor-search-field'
        );

        $this->add_control(
            'search_field_width',
            [
                'label' => esc_html__( 'Width', 'searcher-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-searcher-elementor-search-field-wrapper' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'search_field_height',
            [
                'label' => esc_html__( 'Height', 'searcher-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-searcher-elementor-search-field-wrapper' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_search_field', [
            'html_class' => 'mdp-searcher-elementor-search-field',
            'active_class' => 'mdp-searcher-elementor-search-field:focus',
            'typography_class' => 'mdp-searcher-elementor-search-field, {{WRAPPER}} .mdp-searcher-elementor-search-field:hover::placeholder',
            'include_active_tab' => true,
            'additional_color' => true,
            'include_outline' => true,
            'additional_color_name' => 'Placeholder color',
            'color_class' => 'mdp-searcher-elementor-search-field',
            'color_hover_class' => 'mdp-searcher-elementor-search-field:hover',
            'color_active_class' => 'mdp-searcher-elementor-search-field:focus',
            'additional_color_class' => 'mdp-searcher-elementor-search-field::placeholder',
            'additional_color_hover_class' => 'mdp-searcher-elementor-search-field:hover::placeholder'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Label.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_label() {

        $this->start_controls_section( 'section_style_label', [
            'label' => esc_html__( 'Label', 'searcher-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'show_search_button' => 'yes'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_label',
            'mdp-searcher-elementor-search-field-label'
        );

        $this->generate_typography_tabs_controls( 'section_style_select', [
            'html_class' => 'mdp-searcher-elementor-search-field-label',
            'typography_class' => 'mdp-searcher-elementor-search-field-label',
            'color_class' => 'mdp-searcher-elementor-search-field-label',
            'color_hover_class' => 'mdp-searcher-elementor-search-field-label:hover'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Result Box.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_result_box() {

        $this->start_controls_section( 'section_style_result_box', [
            'label' => esc_html__( 'Result box', 'searcher-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'request_type' => 'async'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_result_box',
            'mdp-searcher-elementor-search-result-box',
                      [ 'top' => 10, 'left' => 10, 'bottom' => 10, 'right' => 10, 'px', true ]
        );

        $this->generate_typography_tabs_controls( 'section_style_result_box', [
            'html_class' => 'mdp-searcher-elementor-search-result-box',
            'typography_class' => 'mdp-searcher-elementor-search-result-box',
            'include_color' => false,
            'additional_color' => true,
            'typography_name' => 'Empty result text typography',
            'additional_color_name' => 'Empty result text color',
            'additional_color_class' => 'mdp-searcher-elementor-search-result-box',
            'additional_color_hover_class' => 'mdp-searcher-elementor-search-result-box:hover'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Result Item.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_result_item() {

        $this->start_controls_section( 'section_style_result_item', [
            'label' => esc_html__( 'Result box item', 'searcher-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'request_type' => 'async'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_result_item',
            'mdp-searcher-elementor-search-result',
                      [],
                      [ 'top' => 0, 'left' => 0, 'bottom' => 10, 'right' => 0, 'px', false ]
        );

        $this->generate_typography_tabs_controls( 'section_style_result_item', [
            'html_class' => 'mdp-searcher-elementor-search-result',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Result Item Post Thumbnail.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_result_item_thumbnail() {

        $this->start_controls_section( 'section_style_result_item_thumbnail', [
            'label' => esc_html__( 'Result box item image', 'searcher-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'request_type' => 'async'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_result_item_thumbnail',
            'mdp-searcher-elementor-search-img-container',
                      [],
                      [ 'top' => 0, 'left' => 0, 'bottom' => 0, 'right' => 25, 'px', false ]
        );

        $this->add_responsive_control(
            'image_height',
            [
                'label' => esc_html__( 'Height', 'searcher-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 50
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-searcher-elementor-search-img-container' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'image_width',
            [
                'label' => esc_html__( 'Width', 'searcher-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 5,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 50
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-searcher-elementor-search-img-container' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_result_item_thumbnail', [
            'html_class' => 'mdp-searcher-elementor-search-img-container',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Result Item Post Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_result_post_title() {

        $this->start_controls_section( 'section_style_result_item_title', [
            'label' => esc_html__( 'Result box item post title', 'searcher-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'request_type' => 'async'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_result_item_title',
            'mdp-searcher-elementor-result-block',
                      [],
                      [ 'top' => 0, 'left' => 0, 'bottom' => 10, 'right' => 0, 'px', false ]
        );

        $this->generate_typography_tabs_controls( 'section_style_result_item_title', [
            'html_class' => 'mdp-searcher-elementor-search-post-title',
            'typography_class' => 'mdp-searcher-elementor-search-post-title',
            'color_class' => 'mdp-searcher-elementor-search-post-title',
            'color_hover_class' => 'mdp-searcher-elementor-result-block:hover .mdp-searcher-elementor-search-post-title'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Result Item Post Excerpt.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_result_post_excerpt() {

        $this->start_controls_section( 'section_style_result_post_excerpt', [
            'label' => esc_html__( 'Result item post excerpt', 'searcher-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'request_type' => 'async'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_result_post_excerpt',
            'mdp-searcher-elementor-search-post-excerpt'
        );

        $this->generate_typography_tabs_controls( 'section_style_result_post_excerpt', [
            'html_class' => 'mdp-searcher-elementor-search-post-excerpt',
            'typography_class' => 'mdp-searcher-elementor-search-post-excerpt',
            'color_class' => 'mdp-searcher-elementor-search-post-excerpt',
            'color_hover_class' => 'mdp-searcher-elementor-result-block:hover .mdp-searcher-elementor-search-post-excerpt'
        ] );

        $this->end_controls_section();

    }

    /**
     * Generate search field label.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_search_field_label( $settings ) {
        $hidden_label_class = $settings['hide_search_bar_label'] === 'yes' ?
            'mdp-searcher-elementor-hide-label' : '';

        return sprintf(
        '<label for="mdp_searcher_search_field_%s" class="mdp-searcher-elementor-search-field-label %s">
                    %s
                </label>',
                $this->get_id(),
                $hidden_label_class,
                esc_html__( $settings['search_bar_label'] )

        );
    }

    /**
     * Generate search field.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_search_field( $settings ) {
        $button_inside = $settings['search_button_position'] === 'inside' &&
            $settings['show_search_button'] === 'yes'  ?
            $this->create_search_button( $settings ) :
            '';

        return sprintf(
        '<div class="mdp-searcher-elementor-search-field-wrapper mdp-searcher-elementor-label-position-%s">
                    %s
                    <input 
                        id="mdp_searcher_search_field_%s" 
                        name="s" 
                        class="mdp-searcher-elementor-search-field" 
                        type="search"
                        autocomplete="off"
                        placeholder="%s"
                    >
                    %s
                </div>',
                esc_attr( $settings['search_bar_label_position'] ),
                $this->create_search_field_label( $settings ),
                $this->get_id(),
                esc_attr( $settings['search_bar_placeholder'] ),
                $button_inside
        );
    }

    /**
     * Create search button icon.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_button_icon( $settings ) {

        if ( empty( $settings['search_button_icon'] ) ) { return ''; }

        return $settings['search_button_icon']['library'] === 'svg' ?
            Icons_Manager::render_uploaded_svg_icon( $settings['search_button_icon']['value'] ) :
            '<i class="'.esc_attr( $settings['search_button_icon']['value'] ).'"></i>';
    }


    /**
     * Generate search button.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_search_button( $settings ) {

        return sprintf(
            '<button class="mdp-searcher-elementor-search-button mdp-searcher-elementor-icon-position-%s">
                        <span class="mdp-searcher-elementor-button-icon">%s</span> %s
                    </button>',
                    esc_attr( $settings['search_button_icon_position'] ),
                    $this->create_button_icon( $settings ),
                    esc_html__( $settings['search_button_content'], 'searcher-elementor' )
        );
    }

    /**
     * Generate post type fields.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function generate_post_type_fields( $post_types ) {
        $post_type_fields = '';
        foreach ( $post_types as $post_type ) {
            $post_type_fields .= sprintf(
                '<input type="hidden" name="mdp_post_type[]" value="%s">',
                       esc_attr( $post_type )
            );
        }

        return $post_type_fields;

    }

    /**
     * Generate search form.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_search_form( $settings ) {
        $search_button = $settings['show_search_button'] === 'yes' &&
            $settings['search_button_position'] !== 'inside' ?
            $this->create_search_button( $settings ) :
            '';

        $search_button_position = $settings['show_search_button'] ?
            sprintf( 'mdp-searcher-elementor-button-position-%s', esc_attr( $settings['search_button_position'] ) )
            : '';

        return sprintf(
            '<form method="get" action="%s" class="mdp-searcher-elementor-search-form %s" role="search">
                        %s
                        %s
                        %s
                        %s
                    </form>',
    get_home_url(),
            $search_button_position,
            $this->generate_post_type_fields( $settings['search_post_types'] ),
            $this->create_search_field( $settings ),
            $search_button,
            $settings['request_type'] === 'async' ? $this->create_result_box() : ''

        );
    }


    private function create_result_box() {
        return sprintf(
            '<div class="mdp-searcher-elementor-search-result-box"></div>'
        );
    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();
        $empty_text = !empty( $settings['empty_result_text'] ) ?
            esc_attr__( $settings['empty_result_text'], 'searcher-elementor' ) :
            esc_attr__( 'Empty search result!', 'searcher-elementor' );

        echo sprintf(
                '<!-- Start Searcher for Elementor WordPress Plugin -->
                        <div class="mdp-searcher-elementor-box"
                        data-search-type="%s"
                        data-post-types="%s"
                        data-search-results-qty="%s"
                        data-form-id="%s"
                        data-post-id="%s"
                        data-request-delay="%s"
                        data-empty-text="%s"
                        >%s</div>
                        <!-- End Searcher for Elementor WordPress Plugin -->',
                esc_attr( $settings['request_type'] ),
                        esc_attr( implode( ',', $settings['search_post_types'] ) ),
                        $settings['request_type'] === 'async' ?
                            esc_attr( $settings['show_results_on_typing']['size'] ) :
                            '',
                        esc_attr( $this->get_id() ),
                        esc_attr( get_the_ID() ),
                        $settings['request_type'] === 'async' ? esc_attr( $settings['request_delay']['size'] ) : '',
                        $settings['request_type'] === 'async' ? $empty_text : '',
                        $this->create_search_form( $settings )
        );

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/searcher';

    }

}
